/*****************************************************************************
*
* File              : TWI_vnc_slave.c
*							 01.05.09, TAB = 3
* AppNote           : AVR311 / 315
*
*
* nderungen        : Enthlt die Fuktionalit eines Slaves.
*							 Luft als (vereinfachter) Slave unabhngig von main():
*							 Es sind nur die Empfangsroutinen aktiv.
*							 Daten, die empfangen werden, werden direkt in einen Ringpuffer beschrieben.
*
****************************************************************************/

#include <avr/interrupt.h>              
#include <avr/io.h>
#include <avr/eeprom.h>
#include <inttypes.h>
#include "TWI_vnc_slave.h"
#include "global.h"

// Verwendet das globale Array vnc_buffer[] mit den Index vnc_head
// und einem Counter fr eingegangene Bytes vnc_bytes_in_buffer

// ----------------------------------------------------------------------------
// Initailisierung fr den Slave Modus
void TWI_init(void)
{
	TWAR = TWI_SLA_ADR;
   TWBR = TWI_TWBR;                             // Set bit rate register (Baudrate). Defined in header file.
// TWSR = TWI_TWPS;                             // Not used. Driver presumes prescaler to be 00.
//	TWDR = 0xFF;                                 // Default content = SDA released.
/*
	TWCR = (1<<TWEN)|                            // TWI Interface enabled.
          (1<<TWIE)|(1<<TWINT)|                 // Enable TWI Interupt and clear the flag.
          (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|      // Prepare to ACK next time the Slave is addressed.
          (0<<TWWC); 
*/
}

/*
// ----------------------------------------------------------------------------
// Wartet bis der Master seine bertragung beendet hat (erkennbar daran, dass TWIE disabled)
// und schaltet dann wieder zurck in den Slave-Modus
void TWI_SLA_restart(void)
{

while (TWCR & 1<<TWIE);									// warten bis der Master fertig ist
TWI_slave_mode = TRUE;									// Flag fr Slave-Modus setzen
																// TWI-Modul wartet als Slave
//TWDR = 0xFF;                                 	// Default content = SDA released.
TWCR = (1<<TWEN)|                            	// TWI Interface enabled.
      (1<<TWIE)|(1<<TWINT)|                 		// Enable TWI Interupt and clear the flag.
      (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|      		// Prepare to ACK next time the Slave is addressed.
      (0<<TWWC); 
}    
 */

/* 

// ****************************************************************************
Call this function to send a prepared message. The first byte must contain the slave address and the
read/write bit. Consecutive bytes contain the data to be sent, or empty locations for data to be read
from the slave. Also include how many bytes that should be sent/read including the address byte.
The function will hold execution (loop) until the TWI_ISR has completed with the previous operation,
then initialize the next operation and return.
// ****************************************************************************
void TWI_MA_Start_Transceiver_With_Data( uint8_t *msg, uint8_t msgSize )
{
   uint8_t temp;

// while ( TWI_Transceiver_Busy() );           	// Wait until TWI is ready for next transmission.

	TWI_slave_mode = FALSE;								// Flag fr Master-Modus
	
   TWI_msgSize = msgSize;                      	// Number of data to transmit.
   TWI_MA_buf[0]  = msg[0];                    	// Store slave address with R/W setting.
   if (!( msg[0] & 0x01))						     	// If it is a write operation, then also copy data.
   {
   for ( temp = 1; temp < msgSize; temp++ )
      TWI_MA_buf[ temp ] = msg[ temp ];
   }
 
   TWCR = (1<<TWEN)|                            // TWI Interface enabled.
          (1<<TWIE)|(1<<TWINT)|                 // Enable TWI Interupt and clear the flag.
          (0<<TWEA)|(1<<TWSTA)|(0<<TWSTO)|      // Initiate a START condition.
          (0<<TWWC);     
}
*/



/****************************************************************************
This function is the Interrupt Service Routine (ISR), and called when the TWI interrupt is triggered;
that is whenever a TWI event has occurred. This function should not be called directly from the main
application.
****************************************************************************/

ISR(TWI_vect)
{
	static uint8_t idx;

//if (TWI_slave_mode)
//	{
	switch (TWSR)
	{
// TWI Slave Transmitter status codes				// ###############################################

   case TWI_STX_ADR_ACK:            				// Own SLA+R has been received; ACK has been returned
		idx = 0;                       				// buffer pointer immer auf das 1. Element

// der Slave hat ein Byte gesendet, der Master mit ACK quittiert
   case TWI_STX_DATA_ACK:           				// Data byte in TWDR has been transmitted; ACK has been received
																// Es werden solange Daten gesendet, bis der Master mit NACK beendet
																// damit kann der Pointer aus dem Array hinauslaufen. 
//		if (idx < 10) TWDR = ptr[idx++];				// man darf ber die Grenzen eines Arrays hinaus lesen 
		if (idx < 4) 
			{
			TWDR = VNC_tx.byte[idx % 0x04];			// gibt binr die ANzahl der bertragenen Datenblcke zurck
			idx++;
			}
/*		else if (idx > 3 && idx < 8)
			{
			TWDR = VNC_fs.byte[idx % 0x04];			// gibt die letzte Dateigre zurck
			idx++;
			}
*/
		else if (idx > 3 && idx < 10)					// gibt die Zeit zurck
			{
			TWDR = time[idx - 4];						// bei der Jahresangabe muss 20 subtrahiert werden !!
			idx++;
			}
		else if (idx == 10)
			{
			TWDR = VNC_save_time;
			idx++;
			}
		else if (idx == 11) 
			{
			TWDR = b_incr_filename;
			idx++;
			}

		else TWDR = 255;									// alle weiteren Anfragen werden mit 0xFF beantwortet !


      TWCR =(1<<TWEN)|                         	// TWI Interface enabled
            (1<<TWIE)|(1<<TWINT)|              	// Enable TWI Interupt and clear the flag to send byte
            (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|   	// Send ACK
            (0<<TWWC);                         	//  
      break;


// TWI Slave Receiver status codes					// ###############################################

// der Slave ist im read-Modus und hat gerade seine eigene Adresse empfangen
// eigene Slave-Adresse + write empfangen und mit ACK quittiert
   case TWI_SRX_ADR_ACK: 								// Own SLA+W has been received ACK has been returned
		TWCR = (1<<TWEN)|                         // TWI Interface enabled
             (1<<TWIE)|(1<<TWINT)|              // Enable TWI Interupt and clear the flag to send byte
             (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|   // Send ACK after next reception
             (0<<TWWC);                         // 
		
		break;
			
// Slave im read Modus, Byte empfangen und mit ACK quittiert, nun das Byte ablegen
   case TWI_SRX_ADR_DATA_ACK:       				// Previously addressed with own SLA+W; data has been received; 
																// ACK has been returned
		vnc_buffer[vnc_head++] = TWDR;				// ein Byte wurde empfangen und in den Puffer eingereiht.
		if (vnc_head > (VNC_BUFFER_SIZE -1)) vnc_head = 0; // auf berlauf testen
		vnc_bytes_in_buffer++;							// Counter incrementieren
		if (vnc_head == vnc_tail) close_file(VNC_CLOSE);	// Notbremse, wenn der Puffer von hinten her berrollt wird
		TWCR = (1<<TWEN)|                         // TWI Interface enabled
             (1<<TWIE)|(1<<TWINT)|              // Enable TWI Interupt and clear the flag to send byte
             (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|   // Send ACK after next reception
             (0<<TWWC);                         //  
		break;

// case TWI_STX_DATA_NACK:          				// Data byte in TWDR has been transmitted; NACK has been received. 
																// I.e. this could be the end of the transmission.
//	case TWI_STX_ADR_ACK_M_ARB_LOST: 				// Arbitration lost in SLA+R/W as Master; 
																// own SLA+R has been received; ACK has been returned
// case TWI_STX_DATA_ACK_LAST_BYTE: 				// Last data byte in TWDR has been transmitted (TWEA = 0); 
																// ACK has been received
// case TWI_SRX_ADR_DATA_NACK:      				// Previously addressed with own SLA+W; data has been received; 
																// NOT ACK has been returned
// case TWI_SRX_GEN_DATA_NACK:      				// Previously addressed with general call; 
																// data has been received; NOT ACK has been returned
// case TWI_SRX_GEN_ACK:            				// General call address has been received; ACK has been returned

// case TWI_SRX_GEN_DATA_ACK:       				// Previously addressed with general call; 
																// data has been received; ACK has been returned
// case TWI_SRX_ADR_ACK_M_ARB_LOST: 				// Arbitration lost in SLA+R/W as Master; 
																// own SLA+W has been received; ACK has been returned 
// case TWI_SRX_GEN_ACK_M_ARB_LOST: 				// Arbitration lost in SLA+R/W as Master; 
																// General call address has been received; ACK has been returned
// case TWI_SRX_STOP_RESTART:       				// A STOP condition or repeated START condition has been received 
																// while still addressed as Slave 
//	case TWI_NO_STATE 									// No relevant state information available; TWINT = 0
// case TWI_BUS_ERROR:         						// Bus error due to an illegal START or STOP condition

// im Slave-Modus laufen alle anderen Flle ber default:
   default:     
     
		TWCR = (1<<TWEN)|                         // TWI Interface enabled
             (1<<TWIE)|(1<<TWINT)|              // Enable TWI Interupt and clear the flag to send byte
             (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|   // Send ACK after next reception
             (0<<TWWC);                      
		break; 
	}
}


/*
//	}
	
//else															// TWI-Master-Mode
//	{
#ifdef DEBUG
	TWI_MA_buf[9] = TWSR;								// den TWI-Status im letzten Byte sichern
#endif	
	switch (TWSR)
	{
// MASTER ########################################################################################	

	// General TWI Master status codes				// ###############################################
// ein TWI-(RE)-Start wurde gesendet, nun muss die Adresse folgen
	case TWI_START:              						// START has been transmitted  
   case TWI_REP_START:          						// Repeated START has been transmitted
//      idx = 0;			           						// Set buffer pointer to the start location
	
	// TWI Master Transmitter status codes 
// die Adresse + W wurde gesendet und vom Slave mit ACK quittiert
   case TWI_MTX_ADR_ACK:        						// SLA+W has been transmitted and ACK received

// eine Datenbyte wurde gesendet und vom Slave mit ACK quittiert
   case TWI_MTX_DATA_ACK:       						// Data byte has been transmitted and ACK received
// wenn noch weitere Bytes abgeholt werden sollen ...
      if (1) //(idx < TWI_msgSize)
			{
			TWDR =TWI_MA_buf[idx++];					// Adresse bzw. Daten nach TWDR schreiben
			TWCR =(1<<TWEN)|                       // TWI Interface enabled
               (1<<TWIE)|(1<<TWINT)|            // Enable TWI Interupt and clear the flag to send byte
               (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|	// Send NACK after reception
               (0<<TWWC);
			}

// alle Bytes gelesen, nun ein Stop senden
// den Rest besorgt 'default'
      else                                      // Send STOP after last byte
			{
//			TWI_statusReg.LastTransOK = TRUE;      // Set status bits to completed successfully.
			TWCR = (1<<TWEN)|                      // TWI Interface enabled
             (0<<TWIE)|(1<<TWINT)|            // Disable TWI Interrupt and clear the flag
             (0<<TWEA)|(0<<TWSTA)|(1<<TWSTO)|  // Initiate a STOP condition.
             (0<<TWWC);
			}

			break;

// ein Datenbyte wurde empfangen und vom Master mit ACK quittiert		
	// TWI Master Receiver status codes				// ###############################################
   case TWI_MRX_DATA_ACK:                       // Data byte has been received and ACK transmitted
      TWI_MA_buf[idx++] = TWDR;						// gelesene Daten in den Puffer schreiben

// die Adresse + R wurde gesendet und vom Slave mit ACK quittiert
   case TWI_MRX_ADR_ACK:                        // SLA+R has been transmitted and ACK received

// es folgen noch weiter Bytes
      if (idx < (TWI_msgSize-1) )        			// Detect the last byte to NACK it.
			{
			TWCR =(1<<TWEN)|                       // TWI Interface enabled
               (1<<TWIE)|(1<<TWINT)|            // Enable TWI Interupt and clear the flag to read next byte
               (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| // Send ACK after reception
               (0<<TWWC);
			}
// das folgende zu lesende Byte ist das letzte, danach wird kein ACK (also ein NACK) gesendet
      else                                      // Send NACK after next reception
			{
			TWCR =(1<<TWEN)|                       // TWI Interface enabled
               (1<<TWIE)|(1<<TWINT)|            // Enable TWI Interupt and clear the flag to read next byte
               (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| // Send NACK after reception
               (0<<TWWC);                          
			}    
      break; 

// das letzte Byte wurde empfangen und vom Master mit NACK quittiert
// nun muss der Master ein Stop senden
   case TWI_MRX_DATA_NACK:                      // Data byte has been received and NACK transmitted
      TWI_MA_buf[idx] = TWDR;
 
      TWCR =(1<<TWEN)|                         	// TWI Interface enabled
            (0<<TWIE)|(1<<TWINT)|              	// Disable TWI Interrupt and clear the flag
            (0<<TWEA)|(0<<TWSTA)|(1<<TWSTO)|   	// Initiate a STOP condition.
            (0<<TWWC);
		break; 

	// General TWI Master status codes				// ###############################################     
   case TWI_ARB_LOST:                           // Arbitration lost
      TWCR =(1<<TWEN)|                         	// TWI Interface enabled
            (1<<TWIE)|(1<<TWINT)|              	// Enable TWI Interupt and clear the flag
            (0<<TWEA)|(1<<TWSTA)|(0<<TWSTO)|   	// Initiate a (RE)START condition.
            (0<<TWWC);                            
      break;
*/
// case TWI_MTX_ADR_NACK:                      	// SLA+W has been transmitted and NACK received
// case TWI_MRX_ADR_NACK:                      	// SLA+R has been transmitted and NACK received    
// case TWI_MTX_DATA_NACK:                     	// Data byte has been transmitted and NACK received
// case TWI_NO_STATE;                         	// No relevant state information available; TWINT = 0
// case TWI_BUS_ERROR:                        	// Bus error due to an illegal START or STOP condition
/*
   default:     

                                                // Reset TWI Interface
      TWCR =(1<<TWEN)|                         	// Enable TWI-interface and release TWI pins
            (0<<TWIE)|(1<<TWINT)|              	// Disable Interupt   - nderung: clear Interrupt Flag
            (0<<TWEA)|(0<<TWSTA)|(1<<TWSTO)|   	// No Signal requests - nderung: Stop Condition senden
            (0<<TWWC);

		break;
		}
//	}
}
*/
// -- [ eof ] -----------------------------------------------------------------